// Author: Pete Kirkham
// tiny quad store benchmark.
//
// read in the SQL triple store dump, output as turtle.

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <iostream>

#define KiB (1024)
#define MiB (1024*1024)

#include "utils/string_utils.h"
#include "io/leap_frog_file_reader.h"
#include "io/file_reader.h"
#include "sql/sql_reader.h"
#include "core/triple_writer.h"


template <class file_reader_t> void run_reader (int file, size_t max_read) 
{
    file_reader_t     fr(file);
    
    // for this particular triples.sql file,
    // skip the first 1270 bytes so we're in the SQL dump, and can expect to find "INSERT INTO `triple_all` VALUES"
    if (!fr.seek(1270))
    {
        std::cerr << "error searching for table start" << std::endl;
        exit(1);
    }

    sql_reader<file_reader_t>     reader(fr);
    triple_writer               target(std::cout);

    // read in data
    reader.pipe_to(target, max_read);

    if (reader.total_read() < 8 * MiB)
    {
        std::cout << "#read " << reader.total_read()/KiB << " KiB." << std::endl;
    }
    else
    {
        std::cout << "#read " << reader.total_read()/MiB << " MiB." << std::endl;
    }

    close(file);
}

int main (int narg, char** argvp)
{
    if (narg <  2)
    {
        std::cerr << "The input file name and the number of MiB to read are required." << std::endl;
        return 1;
    }

    size_t max_read(narg > 2 ? atoi(argvp[2]) * MiB : std::numeric_limits<size_t>::max());
    
    bool leap_frog(false);
    
    
    for (int i(1); i < narg; ++i)
    {
        if (strcmp(argvp[i], "-leap-frog") == 0)      leap_frog = true;
    }

    // open the file
    int file(open(argvp[1], O_RDONLY));

    if (!file)
    {
        std::cerr << "failed to open " << argvp[1] << std::endl;
        return 2;
    }
    
    if (leap_frog)
    {
        run_reader<leap_frog_file_reader>(file, max_read);
    }
    else
    {
        run_reader<file_reader>(file, max_read);
    }
    
    return 0;
}
